#!/usr/bin/env python3
"""prestar
"""
#gero@bioinf.uni-leipzig.de
import sys
import os
import subprocess
import argparse

parser = argparse.ArgumentParser(description='DIEGO scanning routine for junction support files (STAR formatted)')
parser.add_argument("-l", "--list", dest='my_list', required=True, type=argparse.FileType('r'),
                    help="sampleName tab-delimiter path/to/fileName")
parser.add_argument("-d", "--anno", dest='my_anno', required=True,
                    help="specifies the path to the annotation "
                         "(bed 6 file of genebounderies with info "
                         "protein_coding;ENSG00000237683.5;AL627309.1 in column four.)")
parser.add_argument("-b", "--intersectBed", dest='my_intersectBed',
                    help="specifies the path to the intersectBed binary ")
parser.add_argument("-o", "--outdir", dest='my_outdir',
                    help="specifies the path to ouput directory "
                         "(tmp files are also created in this directory, they are removed by rm *.rm) (default is '.')")

args = parser.parse_args()
anno = os.path.abspath(args.my_anno)
if args.my_intersectBed:
    intersectBed = os.path.abspath(args.my_intersectBed)
else:
    intersectBed = "intersectBed"
if args.my_outdir:
    outPath = os.path.abspath(args.my_outdir)
else:
    outPath = os.path.abspath(".")
os.chdir(outPath)

sampleH = {}
unionH = {}


def build_datastructure():
    """Build data structure.

    """
    line_counter = 0
    for line in args.my_list:
        line_counter += 1
        line = line.strip()
        if line != "":
            columns = line.strip().split('\t')
            if len(columns) != 2:
                sys.stderr.write("\nThe file specified with '--list' has {} columns in line {}. "
                                 "Please check the file format and make sure each line has two tab delimited columns."
                                 "\n".format(len(columns), line_counter))
                raise RuntimeError
            if columns[0] not in sampleH:
                sampleJunctionH = {}
                myTuple = (columns[1], sampleJunctionH)
                sampleH[columns[0]] = myTuple
            else:
                sys.stderr.write("error sample "+str(columns[0])+" is already present in data structure")
                sys.exit(1)
        else:
            sys.stderr.write("\nWARNING!!: Ignoring empty line {} in the file specified with '--list'. "
                             " \n".format(line_counter))


def transform_data():
    """rtransform data

    """
    for sampleK, sampleV in sampleH.items():
        with open(sampleV[0]) as myStream:
            for line in myStream:
                columns = line.strip().split('\t')
                junctionKey = str(columns[0])+":"+str(columns[1])+"-"+str(columns[2])
                if junctionKey not in unionH:
                    unionH[junctionKey] = columns[3]
                sampleH[sampleK][1][junctionKey] = columns[6]
    for junctionKey in unionH.keys():
        for sampleK, sampleV in sampleH.items():
            if junctionKey not in sampleV[1]:
                sampleH[sampleK][1][junctionKey] = 0


def check_anno():
    """check annoation

    """
    if os.path.exists(anno):
        with open(anno, 'r') as i:
            for line in i:
                if not line.startswith('#'):
                    columns = line.strip().split("\t")
                    if len(columns) != 6:
                        sys.stderr.write("File: '{}' does not have six columns.".format(anno))
                        raise RuntimeError
                    elif len(columns[3].strip().split(";")) != 3:
                        sys.stderr.write("File: '{}' does not have correct format in column four".format(anno))
                        raise RuntimeError
    else:
        sys.stderr.write("Error. Can not find file: '{}'.".format(anno))
        raise RuntimeError


def intersect_with_anno():
    """intersect annotation

    """
    sys.stdout.write("intersecting splice junctions with the provided annotation\n")
    with open('j_p.bed', 'a') as the_file:
        for key in sorted(unionH.keys()):
            columns = key.strip().split(':')
            fields = columns[1].strip().split('-')
            strandInfo = unionH[key]
            if strandInfo == "0":
                the_file.write(columns[0]+"\t"+fields[0]+"\t"+fields[1]+"\tname\t1\t.\n")
            elif strandInfo == "1":
                the_file.write(columns[0]+"\t"+fields[0]+"\t"+fields[1]+"\tname\t1\t+\n")
            elif strandInfo == "2":
                the_file.write(columns[0]+"\t"+fields[0]+"\t"+fields[1]+"\tname\t1\t-\n")
            else:
                sys.stdout.write("ERROR EMPTY STRAND INFO\n")

    subprocess.call(intersectBed + ' -a j_p.bed -b ' + anno +
                    ' -s -wa | sort | uniq -c | awk \'{if ($1>1){print $2"\t"$3"\t"$4"\t"$5"\t"$6"\t"$7}}\' > j_r1.bed',
                    shell=True)
    subprocess.call(intersectBed + ' -a j_p.bed -b ' + anno + ' -v -wa > j_r2.bed', shell=True)
    subprocess.call('cat j_r1.bed j_r2.bed | sort -u > j_r.bed', shell=True)
    subprocess.call('awk -F $\'\t\' \'FNR==NR {a[$1":"$2":"$3]=1; next} '
                    '{if ($1":"$2":"$3 in a){hans+1}else{print $0}}\' j_r.bed j_p.bed > j.bed', shell=True)
    subprocess.call(intersectBed + ' -a j.bed -b ' + anno +
                    ' -f 1.0 -wa -wb | awk \'{split($10,a,";"); split(a[2],b,"."); split(a[3],c,"."); '
                    'print $1":"$2"-"$3"\tN_w\t"b[1]"\t"c[1]}\' > junctions_gene.rm', shell=True)
    subprocess.call('rm j_p.bed j_r1.bed j_r2.bed j_r.bed j.bed', shell=True)


def build_table():
    """build table

    """
    sys.stdout.write("building table\n")
    subprocess.call('echo junction > firstline.rm', shell=True)
    with open('junctions.rm', 'a') as the_file:
        for key in sorted(unionH.keys()):
            the_file.write(str(key) + "\n")
    subprocess.call('echo type > type.rm', shell=True)
    for sampleKey, sampleV in sampleH.items():
        subprocess.call('echo ' + sampleKey + ' > ' + sampleKey + '_firstline.rm', shell=True)
        with open(sampleKey + '_tmp.rm', 'a') as support_file:
            for junctionKey in sorted(sampleV[1].keys()):
                support_file.write(str(sampleV[1][junctionKey]) + "\n")

    subprocess.call('paste firstline.rm type.rm *_firstline.rm | '
                    'awk \'{print $0 "\tgeneID\tgeneName"}\' > junction_table.txt', shell=True)
    subprocess.call('paste junctions.rm *_tmp.rm | sort -u > junction_table_body.rm', shell=True)
    subprocess.call('awk \'FNR==NR {a[$1]=$0; next} {if ($1 in a){ split(a[$1],b,"\t"); '
                    'printf (b[1]"\t"b[2]); for(i=2; i<=NF; i++){printf "\t"$i}; print "\t"b[3]"\t"b[4]}}\' '
                    'junctions_gene.rm junction_table_body.rm >> junction_table.txt', shell=True)
    subprocess.call('rm *.rm', shell=True)


check_anno()
build_datastructure()
transform_data()
intersect_with_anno()
build_table()
sys.stderr.write('Now there should be a file named junction_table.txt in your specified out folder '
                 'that can be used as input for DIEGO.py \n')
